from pelican import Pelican, signals

from .generator import DataGenerator


def get_generators(pelican_object: Pelican):
    """
    Provides custom generator

    :param pelican_object: Pelican
    :return: DataGenerator
    """

    return DataGenerator


def register() -> None:
    """
    Registers custom generator

    :return: None
    """

    signals.get_generators.connect(get_generators)



import json
import pathlib
from typing import Any, Dict, List, Optional

try:
    import tomllib
except ModuleNotFoundError:
    try:
        import tomli as tomllib
    except ModuleNotFoundError:
        pass

try:
    import yaml
except ModuleNotFoundError:
    pass


class DataParser:
    """
    Handles data files
    """

    handlers: Dict[str, List[str]]

    def __init__(self) -> None:
        """
        Constructor

        :return: None
        """

        # Support JSON by default
        handlers = {"json": [".json"]}

        # Detect support for other file formats
        # (1) TOML
        if "tomllib" in globals():
            handlers["toml"] = [".toml"]

        # (2) YAML
        if "yaml" in globals():
            handlers["yaml"] = [".yml", ".yaml"]

        self.handlers = handlers

    def load(self, file: pathlib.Path) -> Optional[Dict[str, Any]]:
        """
        Loads data from FILE

        :param file: pathlib.Path Filepath
        :return: Dict[str, Any] | None
        """

        # Determine file extension
        extension = file.suffix.lower()

        for handler, extensions in self.handlers.items():
            if extension in extensions:
                try:
                    return getattr(self, f"load_{handler}")(file)
                except Exception:
                    pass

    def load_json(self, json_file: pathlib.Path) -> Dict[str, Any]:
        """
        Loads data from JSON file

        :param json_file: pathlib.Path Filepath
        :return: Dict[str, Any] Data
        """

        with json_file.open("r") as file:
            return json.load(file)

    def load_yaml(self, yaml_file: pathlib.Path) -> Dict[str, Any]:
        """
        Loads data from YAML file

        :param yaml_file: pathlib.Path Filepath
        :return: Dict[str, Any] Data
        """

        with yaml_file.open("r") as file:
            return yaml.safe_load(file)

    def load_toml(self, toml_file: pathlib.Path) -> Dict[str, Any]:
        """
        Loads data from TOML file

        :param toml_file: pathlib.Path Filepath
        :return: Dict[str, Any] Data
        """

        with toml_file.open("rb") as file:
            return tomllib.load(file)



import sys
import os
import logging
import pathlib

from pelican.generators import Generator

from .parser import DataParser

log = logging.getLogger(__name__)
log.setLevel(logging.INFO)


class DataGenerator(Generator):
    """
    Load data from files
    """

    def __init__(
        self,
        context,
        settings,
        path,
        theme,
        output_path,
        readers_cache_name="",
        **kwargs,
    ):
        super().__init__(
            context,
            settings,
            path,
            theme,
            output_path,
            readers_cache_name=readers_cache_name,
            **kwargs,
        )

        log.info("PLUGIN: pelican-data-files was successfully loaded")
        self.settings.setdefault("DATA_FILES_DIR", "data")

    def generate_context(self) -> None:
        """
        Generate context from data files

        :return: None
        """

        data_dir = pathlib.Path(self.settings["DATA_FILES_DIR"])

        # Turn path into absolute if not already
        if not data_dir.is_absolute():
            data_dir = pathlib.Path(self.settings["PATH"]).joinpath(data_dir)

        # Check if path exists
        if not data_dir.exists():
            log.error("pelican-data-files: DATA_FILES_DIR path doesn't exists.")
            sys.exit(1)

        if not data_dir.is_dir():
            log.error("pelican-data-files: DATA_FILES_DIR path isn't a directory.")
            sys.exit(1)

        # Initialize parser object
        obj = DataParser()

        # Sort files by modified time
        files = sorted(data_dir.iterdir(), key=os.path.getmtime, reverse=True)

        # Create data array
        buffer = []

        for file in files:
            # Skip duplicate files
            if file.stem in buffer:
                continue

            # Load data from file
            data = obj.load(file)

            # Check whether file (type) is supported
            if data is None:
                # Report back
                log.info(f"{file.name} wasn't loaded.")

                # Move on to next file
                continue

            # Determine context variable
            name = file.stem.replace(".", "_").upper()

            # Add data to context
            self.context[f"DATA_{name}"] = data

            # Remember filename
            buffer.append(file.stem)

            # Report back
            log.info(f"{file.name} was loaded.")



import argparse
import shutil
import sys
from pathlib import Path


def _err(msg, die=None):
    """Print an error message and exits if an exit code is given"""
    sys.stderr.write(f"ERROR: {msg}\n")
    if die:
        sys.exit(die if type(die) is int else 1)


try:
    import pelican
except ImportError:
    _err(
        "Cannot import pelican.\nYou must "
        "install Pelican in order to run this script.",
        1,
    )

# where data files are stored in pelican-data-files's compatible theme
SRC_DATA_DIR = "data"

# where data file are copied in pelican project directory
DEST_DATA_DIR = "content/data"

PELICAN_ROOT_DIR = Path(pelican.__file__).absolute().parent
PELICAN_THEMES_DIR = PELICAN_ROOT_DIR.joinpath("themes")


def _parse_args():
    """Parse and return cli arguments"""
    parser = argparse.ArgumentParser(
        description="""List and fetch data files from compatible themes"""
    )

    parser.add_argument(
        "-l",
        "--list",
        action="store_true",
        help="list all themes compatible with this plugin",
    )

    parser.add_argument(
        "-f",
        "--fetch",
        type=str,
        metavar="THEME_NAME",
        help="fetch theme's data files",
    )

    # parser.add_argument(
    #     "--force",
    #     action="store_true",
    #     help="overwrite files already present in your project",
    # )

    # if no args parsed, set --help
    return parser.parse_args(None if sys.argv[1:] else ["--help"])


def _get_themes():
    return [theme for theme in PELICAN_THEMES_DIR.iterdir() if theme.is_dir()]


def _get_compatible_themes(themes):
    return [theme for theme in themes if _is_compatible(theme)]


def _is_compatible(theme):
    return theme.joinpath(SRC_DATA_DIR).exists()


def _fetch_files(theme):
    try:
        shutil.copytree(
            theme.joinpath(SRC_DATA_DIR), Path.cwd().joinpath(DEST_DATA_DIR)
        )
    except FileExistsError:
        _err(
            f"Cannot copy '{theme.name}' files, "
            f"directory '{DEST_DATA_DIR}' already exists.",
            die=2,
        )
    else:
        print(f"'{theme.name}' files successfully fetched.")


def main():
    """Main function, called by entrypoint"""
    args = _parse_args()

    if args.list:
        for theme in _get_compatible_themes(_get_themes()):
            print(f"{theme.name}")
        sys.exit(0)

    if args.fetch:
        theme = PELICAN_THEMES_DIR.joinpath(args.fetch)
        if theme.exists():
            if _is_compatible(theme):
                _fetch_files(theme)
                sys.exit(0)
            else:
                _err(f"'{args.fetch}' is not a compatible theme.", die=2)
        else:
            _err(f"'{args.fetch}' is not an existing theme.", die=2)
